@@ -1,6 +1,6 @@
 <?php
 /**
- * $Horde: turba/browse.php,v 1.76.2.21 2006/03/02 17:18:56 chuck Exp $
+ * $Horde: turba/browse.php,v 1.76.2.23 2006/03/10 05:30:06 selsky Exp $
  *
  * Turba: Copyright 2000-2005 Charles J. Hagenbuch <chuck@horde.org>
  *
@@ -273,6 +273,13 @@ if (isset($driver)) {
         }
         break;
     }
+
+    // We might get here from the search page but are not allowed to browse
+    // the current address book.
+    if ($actionID && empty($cfgSources[$source]['browse'])) {
+        header('Location: ' . Horde::applicationUrl($prefs->getValue('initial_page'), true));
+        exit;
+    }
 }
 
 $templates = array();
