@@ -1,6 +1,6 @@
 <?php
 /**
- * $Horde: turba/data.php,v 1.70.4.5 2006/01/01 21:29:15 jan Exp $
+ * $Horde: turba/data.php,v 1.70.4.8 2006/08/22 15:44:17 jan Exp $
  *
  * Copyright 2001-2006 Jan Schneider <jan@horde.org>
  *
@@ -212,7 +212,7 @@ case 'export':
     case EXPORT_VCARD:
         $cards = array();
         foreach ($contacts as $contact) {
-            $cards[] =& Turba_Driver::tovCard($contact);
+            $cards[] = Turba_Driver::tovCard($contact);
         }
 
         $vcard = &Horde_Data::singleton('vcard');
@@ -290,11 +290,19 @@ if (!$error && !empty($driver)) {
 
 /* We have a final result set. */
 if (is_array($next_step)) {
+    /* Create a category manager. */
+    require_once 'Horde/Prefs/CategoryManager.php';
+    $cManager = new Prefs_CategoryManager();
+    $categories = $cManager->get();
+
     /* Create a Turba storage instance. */
     $dest = $_SESSION['import_data']['target'];
     $storage = &Turba_Driver::singleton($dest);
     if (is_a($storage, 'PEAR_Error')) {
         $notification->push(sprintf(_("Failed to access the address book: %s"), $storage->getMessage()), 'horde.error');
+    } elseif (!count($next_step)) {
+        $notification->push(sprintf(_("The %s file didn't contain any contacts."),
+                                    $file_types[$_SESSION['import_data']['format']]), 'horde.error');
     } else {
         /* Purge old address book if requested. */
         if ($_SESSION['import_data']['purge']) {
@@ -337,6 +345,12 @@ if (is_array($next_step)) {
                     $error = true;
                     break;
                 }
+
+                if (!empty($row['category']) &&
+                    !in_array($row['category'], $categories)) {
+                    $cManager->add($row['category']);
+                    $categories[] = $row['category'];
+                }
             }
         }
         if (!$error) {
