@@ -4,7 +4,7 @@
  * various directory search drivers.  It includes functions for searching,
  * adding, removing, and modifying directory entries.
  *
- * $Horde: turba/lib/Driver.php,v 1.57.2.22 2006/03/03 16:45:54 jan Exp $
+ * $Horde: turba/lib/Driver.php,v 1.57.2.32 2006/10/31 17:05:45 jan Exp $
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Jon Parise <jon@csh.rit.edu>
@@ -435,6 +435,9 @@ class Turba_Driver {
         $criteria = $this->map['__key'];
 
         $objects = $this->_read($criteria, $objectIds, array_values($this->fields));
+        if (is_a($objects, 'PEAR_Error')) {
+            return $objects;
+        }
         if (!is_array($objects)) {
             $result = PEAR::raiseError(_("Requested object not found."));
             return $result;
@@ -673,42 +676,57 @@ class Turba_Driver {
     /**
      * Exports a given Turba_Object as an iCalendar vCard.
      *
-     * @param Turba_Object $object  A Turba_Object.
+     * @param Turba_Object $object    A Turba_Object.
+     * @param string       $version   The vcard version to produce.
      *
      * @return Horde_iCalendar_vcard  A Horde_iCalendar_vcard object.
      */
-    function tovCard($object)
+    function tovCard($object, $version = '2.1')
     {
         require_once 'Horde/iCalendar/vcard.php';
 
         $hash = $object->getAttributes();
-        $iCal = &new Horde_iCalendar();
-        $vcard = Horde_iCalendar::newComponent('vcard', $iCal);
+        $vcard = &new Horde_iCalendar_vcard();
+        $formattedname = false;
 
         foreach ($hash as $key => $val) {
             switch ($key) {
             case 'name':
                 $vcard->setAttribute('FN', $val);
+                $formattedname = true;
                 break;
-
             case 'nickname':
+            case 'alias':
                 $vcard->setAttribute('NICKNAME', $val);
                 break;
-
             case 'homePhone':
-                $vcard->setAttribute('TEL', $val, array('TYPE'=>'HOME'));
+                if ($version == '2.1') {
+                    $vcard->setAttribute('TEL', $val, array('HOME' => null));
+                } else {
+                    $vcard->setAttribute('TEL', $val, array('TYPE'=>'HOME'));
+                }
                 break;
-
             case 'workPhone':
-                $vcard->setAttribute('TEL', $val, array('TYPE'=>'WORK'));
+                if ($version == '2.1') {
+                    $vcard->setAttribute('TEL', $val, array('WORK' => null));
+                } else {
+                    $vcard->setAttribute('TEL', $val, array('TYPE'=>'WORK'));
+                }
                 break;
-
             case 'cellPhone':
-                $vcard->setAttribute('TEL', $val, array('TYPE'=>'CELL'));
+                if ($version == '2.1') {
+                    $vcard->setAttribute('TEL', $val, array('CELL' => null));
+                } else {
+                    $vcard->setAttribute('TEL', $val, array('TYPE'=>'CELL'));
+                }
                 break;
 
             case 'fax':
-                $vcard->setAttribute('TEL', $val, array('TYPE'=>'FAX'));
+                if ($version == '2.1') {
+                    $vcard->setAttribute('TEL', $val, array('FAX' => null));
+                } else {
+                    $vcard->setAttribute('TEL', $val, array('TYPE'=>'FAX'));
+                }
                 break;
 
             case 'email':
@@ -719,10 +737,6 @@ class Turba_Driver {
                 $vcard->setAttribute('TITLE', $val);
                 break;
 
-            case 'company':
-                $vcard->setAttribute('ORG', $val);
-                break;
-
             case 'notes':
                 $vcard->setAttribute('NOTE', $val);
                 break;
@@ -737,38 +751,86 @@ class Turba_Driver {
             }
         }
 
-        if (!empty($hash['lastname']) || !empty($hash['firstname'])) {
-            $a = array(
-                VCARD_N_FAMILY => isset($hash['lastname']) ? $hash['lastname'] : '',
-                VCARD_N_GIVEN  => isset($hash['firstname']) ? $hash['firstname'] : '',
-                VCARD_N_ADDL   => '',
-                VCARD_N_PREFIX => '',
-                VCARD_N_SUFFIX => '',
-            );
-            $vcard->setAttribute('N', implode(';', $a), array(), false, $a);
+        // No explicit firstname/lastname in data source: we have to guess.
+        if (!isset($this->map['lastname'])) {
+            $i = strpos($hash['name'], ',');
+            if (is_int($i)) {
+                // Assume Last, First
+                $hash['lastname'] = String::substr($hash['name'], 0, $i);
+                $hash['firstname'] = trim(String::substr($hash['name'], $i + 1));
+            } elseif (is_int(strpos($hash['name'], ' '))) {
+                // Assume everything after last space as lastname
+                $i = strrpos($hash['name'], ' ');
+                $hash['lastname'] = trim(String::substr($hash['name'], $i + 1));
+                $hash['firstname'] = String::substr($hash['name'], 0, $i);
+            } else {
+                $hash['lastname'] = $hash['name'];
+                $hash['fistname'] = '';
+            }
         }
 
+        $a = array(
+            VCARD_N_FAMILY => isset($hash['lastname']) ? $hash['lastname'] : '',
+            VCARD_N_GIVEN  => isset($hash['firstname']) ? $hash['firstname'] : '',
+            VCARD_N_ADDL   => isset($hash['initials']) ? $hash['initials'] : '',
+            VCARD_N_PREFIX => isset($hash['salutation']) ? $hash['salutation'] : '',
+            VCARD_N_SUFFIX => '',
+        );
+        $vcard->setAttribute('N', implode(';', $a), array(), false, $a);
+
+        if (!$formattedname) {
+            if (empty($hash['firstname'])) {
+                $vcard->setAttribute('FN', $hash['lastname']);
+            } else {
+                $vcard->setAttribute('FN', $hash['firstname'] . ' '
+                                           . $hash['lastname']);
+            }
+        }
+
+        /* We can't know if this particular turba source uses a single Address
+         * field or multiple for street/city/province/postcode/country. Try to
+         * deal with both. */
+        if (isset($hash['homeAddress']) && !isset($hash['homeStreet'])) {
+            $hash['homeStreet'] = $hash['homeAddress'];
+        }
         $a = array(
             VCARD_ADR_POB      => '',
             VCARD_ADR_EXTEND   => '',
-            VCARD_ADR_STREET   => isset($hash['homeAddress']) ? $hash['homeAddress'] : '',
+            VCARD_ADR_STREET   => isset($hash['homeStreet']) ? $hash['homeStreet'] : '',
             VCARD_ADR_LOCALITY => isset($hash['homeCity']) ? $hash['homeCity'] : '',
             VCARD_ADR_REGION   => isset($hash['homeProvince']) ? $hash['homeProvince'] : '',
             VCARD_ADR_POSTCODE => isset($hash['homePostalCode']) ? $hash['homePostalCode'] : '',
             VCARD_ADR_COUNTRY  => isset($hash['homeCountry']) ? $hash['homeCountry'] : '',
         );
-        $vcard->setAttribute('ADR', implode(';', $a), array('TYPE' => 'HOME'), true, $a);
 
+        if ($version == '2.1') {
+            $vcard->setAttribute('ADR', implode(';', $a), array('HOME' => null), true, $a);
+        } else {
+            $vcard->setAttribute('ADR', implode(';', $a), array('TYPE' => 'HOME'), true, $a);
+        }
+
+	$a = array(0 => isset($hash['company']) ? $hash['company'] : '',
+                   1 => isset($hash['department']) ? $hash['department'] : '');
+	$vcard->setAttribute('ORG', implode(';', $a), array(), false, $a);
+
+        if (isset($hash['workAddress']) && !isset($hash['workStreet'])) {
+            $hash['workStreet'] = $hash['workAddress'];
+        }
         $a = array(
             VCARD_ADR_POB      => '',
             VCARD_ADR_EXTEND   => '',
-            VCARD_ADR_STREET   => isset($hash['workAddress']) ? $hash['workAddress'] : '',
+            VCARD_ADR_STREET   => isset($hash['workStreet']) ? $hash['workStreet'] : '',
             VCARD_ADR_LOCALITY => isset($hash['workCity']) ? $hash['workCity'] : '',
             VCARD_ADR_REGION   => isset($hash['workProvince']) ? $hash['workProvince'] : '',
             VCARD_ADR_POSTCODE => isset($hash['workPostalCode']) ? $hash['workPostalCode'] : '',
             VCARD_ADR_COUNTRY  => isset($hash['workCountry']) ? $hash['workCountry'] : '',
         );
-        $vcard->setAttribute('ADR', implode(';', $a), array('TYPE' => 'WORK'), true, $a);
+
+        if ($version == '2.1') {
+            $vcard->setAttribute('ADR', implode(';', $a), array('WORK' => null), true, $a);
+        } else {
+            $vcard->setAttribute('ADR', implode(';', $a), array('TYPE' => 'WORK'), true, $a);
+        }
 
         return $vcard;
     }
@@ -799,7 +861,9 @@ class Turba_Driver {
 
             switch ($item['name']) {
             case 'FN':
-                $hash['name'] = $item['value'];
+                if (isset($this->fields['name'])) {
+                    $hash['name'] = $item['value'];
+                }
                 break;
 
             case 'N':
@@ -810,6 +874,7 @@ class Turba_Driver {
 
             case 'NICKNAME':
                 $hash['nickname'] = $item['value'];
+                $hash['alias'] = $item['value'];
                 break;
 
             // We use LABEL but also support ADR.
@@ -903,12 +968,12 @@ class Turba_Driver {
                         }
                     }
                 } else {
-                    if (isset($item['params']['HOME'])) {
+                    if (isset($item['params']['CELL'])) {
+                        $hash['cellPhone'] = $item['value'];
+                    } elseif (isset($item['params']['HOME'])) {
                         $hash['homePhone'] = $item['value'];
                     } elseif (isset($item['params']['WORK'])) {
                         $hash['workPhone'] = $item['value'];
-                    } elseif (isset($item['params']['CELL'])) {
-                        $hash['cellPhone'] = $item['value'];
                     } else {
                         $hash['homePhone'] = $item['value'];
                     }
@@ -926,7 +991,11 @@ class Turba_Driver {
                 break;
 
             case 'ORG':
-                $hash['company'] = $item['value'];
+		// The VCARD 2.1 specification requires the presence of two
+		// SEMI-COLON separated fields: Organizational Name and
+		// Organizational Unit. Additional fields are optional.
+		$hash['company'] = !empty($item['values'][0]) ? $item['values'][0] : '';
+		$hash['department'] = !empty($item['values'][1]) ? $item['values'][1] : '';
                 break;
 
             case 'NOTE':
@@ -943,6 +1012,17 @@ class Turba_Driver {
             }
         }
 
+        /* Ensure we have a valid name field. */
+        if (isset($this->fields['name']) && empty($hash['name'])) {
+            $hash['name'] = $hash['firstname'] . ' ' . $hash['lastname'];
+            if (trim($hash['name']) === '') {
+                $hash['name'] = $vcard->getAttributeDefault('FN');
+            }
+        }
+        if (isset($this->fields['lastname']) && empty($hash['lastname'])) {
+            $hash['lastname'] = $vcard->getAttributeDefault('FN');
+        }
+
         return $hash;
     }
 
