@@ -2,7 +2,7 @@
 /**
  * Turba directory driver implementation for PHP's LDAP extension.
  *
- * $Horde: turba/lib/Driver/ldap.php,v 1.54.4.12 2005/11/10 12:44:30 mdjukic Exp $
+ * $Horde: turba/lib/Driver/ldap.php,v 1.54.4.16 2006/09/28 05:13:15 chuck Exp $
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Jon Parise <jon@csh.rit.edu>
@@ -287,23 +287,17 @@ class Turba_Driver_ldap extends Turba_Driver {
         $oldres = @ldap_read($this->_ds, String::convertCharset($object_id, NLS::getCharset(), $this->_params['charset']), $filter, array_merge(array_keys($attributes), array('objectclass')));
         $info = ldap_get_attributes($this->_ds, ldap_first_entry($this->_ds, $oldres));
 
-        /* Check if we need to rename the object. */
         if ($this->_params['version'] == 3 &&
-            String::lower($this->_makeKey($attributes)) != String::lower($object_id)) {
-            if (isset($this->_params['dn']) && is_array($this->_params['dn']) && count($this->_params['dn'])) {
-                $pairs = array();
-                foreach ($this->_params['dn'] as $param) {
-                    if (isset($attributes[$param])) {
-                        $pairs[] = array($param, $attributes[$param]);
-                    }
-                }
-                $newrdn = $this->_quoteDN($pairs);
-            } else {
+            String::lower(str_replace(array(',', '"'), array('\\2C', ''), $this->_makeKey($attributes))) !=
+            String::lower(str_replace(',', '\\2C', $object_id))) {
+            /* Need to rename the object. */
+            $newrdn = $this->_makeRDN($attributes);
+            if ($newrdn == '') {
                 return PEAR::raiseError(_("Missing DN in LDAP source configuration."));
             }
 
             if (ldap_rename($this->_ds, String::convertCharset($object_id, NLS::getCharset(), $this->_params['charset']),
-                    String::convertCharset($newrdn, NLS::getCharset(), $this->_params['charset']), $this->_params['root'], true)) {
+                            String::convertCharset($newrdn, NLS::getCharset(), $this->_params['charset']), $this->_params['root'], true)) {
                 $object_id = $newrdn . ',' . $this->_params['root'];
             } else {
                 return PEAR::raiseError(sprintf(_("Failed to change name: (%s) %s; Old DN = %s, New DN = %s, Root = %s"),
@@ -311,10 +305,9 @@ class Turba_Driver_ldap extends Turba_Driver {
             }
         }
 
-        /* The attributes in the attributes array are all lower case
-         * while they are mixedCase in the search result. So convert
-         * the keys to lower. */
+        /* Work only with lowercase keys. */
         $info = array_change_key_case($info, CASE_LOWER);
+        $attributes = array_change_key_case($attributes, CASE_LOWER);
 
         foreach ($info as $key => $value) {
             $var = $info[$key];
@@ -335,7 +328,7 @@ class Turba_Driver_ldap extends Turba_Driver {
             }
         }
 
-        unset($attributes[$object_key]);
+        unset($attributes[String::lower($object_key)]);
         $this->_encodeAttributes($attributes);
         $attributes = array_filter($attributes, array($this, '_emptyAttributeFilter'));
 
@@ -351,6 +344,30 @@ class Turba_Driver_ldap extends Turba_Driver {
         }
     }
 
+    /**
+     * Build a RDN based on a set of attributes and what attributes
+     * make a RDN for the current source.
+     *
+     * @param array $attributes The attributes (in driver keys) of the
+     *                          object being added.
+     *
+     * @return string  The RDN for the new object.
+     */
+    function _makeRDN($attributes)
+    {
+        if (!is_array($this->_params['dn'])) {
+            return '';
+        }
+
+        $pairs = array();
+        foreach ($this->_params['dn'] as $param) {
+            if (isset($attributes[$param])) {
+                $pairs[] = array($param, $attributes[$param]);
+            }
+        }
+        return $this->_quoteDN($pairs);
+    }
+
     /**
      * Build a DN based on a set of attributes and what attributes
      * make a DN for the current source.
@@ -362,18 +379,7 @@ class Turba_Driver_ldap extends Turba_Driver {
      */
     function _makeKey($attributes)
     {
-        $dn = '';
-        if (is_array($this->_params['dn'])) {
-            $pairs = array();
-            foreach ($this->_params['dn'] as $param) {
-                if (isset($attributes[$param])) {
-                    $pairs[] = array($param, $attributes[$param]);
-                }
-            }
-            $dn = $this->_quoteDN($pairs);
-        }
-
-        return $dn . ',' . $this->_params['root'];
+        return $this->_makeRDN($attributes) . ',' . $this->_params['root'];
     }
 
     /**
@@ -627,14 +633,9 @@ class Turba_Driver_ldap extends Turba_Driver {
      */
     function _isPostalAddress($attribute)
     {
-        $syntax = $this->_getSyntax($attribute);
-
-        /** LDAP postal address syntax
+        /* LDAP postal address syntax is
          * 1.3.6.1.4.1.1466.115.121.1.41 */
-        if ($syntax == "1.3.6.1.4.1.1466.115.121.1.41") {
-            return true;
-        }
-        return false;
+        return $this->_getSyntax($attribute) == '1.3.6.1.4.1.1466.115.121.1.41';
     }
 
     /**
@@ -652,8 +653,8 @@ class Turba_Driver_ldap extends Turba_Driver {
         if (is_a($schema, 'PEAR_Error')) {
             return $schema;
         }
-        $attv = $schema->get('attribute', $att);
 
+        $attv = $schema->get('attribute', $att);
         if (isset($attv['syntax'])) {
             return $attv['syntax'];
         } else {
