@@ -3,7 +3,7 @@
  * Turba directory driver implementation for PHP's PEAR database abstraction
  * layer.
  *
- * $Horde: turba/lib/Driver/sql.php,v 1.59.10.15 2006/01/18 19:50:25 ben Exp $
+ * $Horde: turba/lib/Driver/sql.php,v 1.59.10.16 2006/05/08 14:21:00 jan Exp $
  *
  * @author  Jon Parise <jon@csh.rit.edu>
  * @package Turba
@@ -191,6 +191,7 @@ class Turba_Driver_sql extends Turba_Driver {
         $result = $this->_db->getAll($query, $values);
         if (is_a($result, 'PEAR_Error')) {
             Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+            return $result;
         }
 
         $results = array();
