@@ -3,7 +3,7 @@
  * The Turba_ListView:: class provides an interface for objects that
  * visualize Turba_lists.
  *
- * $Horde: turba/lib/ListView.php,v 1.17.10.9 2005/12/08 18:36:59 mrubinsk Exp $
+ * $Horde: turba/lib/ListView.php,v 1.17.10.11 2006/08/14 17:29:18 jan Exp $
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Jon Parise <jon@csh.rit.edu>
@@ -152,7 +152,7 @@ class Turba_ListView {
 
     function display()
     {
-        global $prefs, $conf, $source, $add_source_options;
+        global $prefs, $conf, $source, $copymove_source_options;
 
         $driver = &Turba_Driver::singleton($source);
         if (!is_a($driver, 'PEAR_Error')) {
@@ -194,7 +194,7 @@ class Turba_ListView {
             require_once 'Horde/Variables.php';
             $vars = &Variables::getDefaultVariables();
             $crit = array();
-            if ($_SESSION['turba_search_mode'] == 'advanced') {
+            if ($_SESSION['turba']['search_mode'] == 'advanced') {
                 $map = $driver->getCriteria();
                 foreach ($map as $key => $value) {
                     if ($key != '__key') {
