@@ -8,7 +8,7 @@ define('TURBA_VFS_PATH', '.horde/turba/documents');
 /**
  * Turba Base Class.
  *
- * $Horde: turba/lib/Turba.php,v 1.59.4.22 2006/02/03 00:48:52 jan Exp $
+ * $Horde: turba/lib/Turba.php,v 1.59.4.25 2006/08/25 13:16:39 jan Exp $
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Jon Parise <jon@horde.org>
@@ -66,10 +66,10 @@ class Turba {
                 if (!is_a($addresses, 'PEAR_Error')) {
                     $mail_link = $addresses[$i . ':' . $j];
                     if (is_a($mail_link, 'PEAR_Error')) {
-                        $mail_link = 'mailto:' . $address;
+                        $mail_link = 'mailto:' . urlencode($email_val);
                     }
                 } else {
-                    $mail_link = 'mailto:' . $address;
+                    $mail_link = 'mailto:' . urlencode($email_val);
                 }
 
                 $email_value = Horde::link($mail_link, $email_val) . htmlspecialchars($email_val) . '</a>';
@@ -500,6 +500,9 @@ class Turba {
     function updateShare($name, $params)
     {
         $share = &$GLOBALS['turba_shares']->getShare($name);
+        if (is_a($share, 'PEAR_Error')) {
+            return $share;
+        }
         $name = $share->get('name');
         foreach ($params as $key => $value) {
             $share->set($key, $value);
