@@ -2,7 +2,7 @@
 /**
  * Turba external API interface.
  *
- * $Horde: turba/lib/api.php,v 1.120.2.26 2006/03/06 13:54:04 jan Exp $
+ * $Horde: turba/lib/api.php,v 1.120.2.36 2006/10/23 16:58:39 karsten Exp $
  *
  * This file defines Turba's external API interface. Other applications can
  * interact with Turba through this API.
@@ -97,7 +97,7 @@ $_services['deleteField'] = array(
 );
 
 $_services['getField'] = array(
-    'args' => array('address' => 'string', 'field' => 'string', 'sources' => '{urn:horde}stringArray', 'strict' => 'boolean'),
+    'args' => array('address' => 'string', 'field' => 'string', 'sources' => '{urn:horde}stringArray', 'strict' => 'boolean', 'multiple' => 'boolean'),
     'type' => '{urn:horde}stringArray',
 );
 
@@ -155,7 +155,6 @@ $_services['hasComments'] = array(
     'type' => 'boolean'
 );
 
-
 function _turba_commentCallback($id)
 {
     if (!$GLOBALS['conf']['comments']['allow']) {
@@ -389,30 +388,36 @@ function _turba_browse($path = '', $properties = array())
  *
  * @return array  An array of UIDs for all contacts the user can access.
  */
-function _turba_list($source = '')
+function _turba_list($source = null)
 {
     require_once dirname(__FILE__) . '/base.php';
 
-    global $cfgSources;
+    global $cfgSources, $prefs;
 
-    // Get default address book from user preferences.
+    /* Get default address book from user preferences. */
     if (empty($source)) {
-        $source = $GLOBALS['prefs']->getValue('default_dir');
+        $source = $prefs->getValue('default_dir');
+        /* On new installations default_dir is not set. */
+        /* Use first source instead. */
+        if (empty($source)) {
+            $source = array_keys($cfgSources);
+            $source = $source[0];
+        }
     }
 
     if (empty($source) || !isset($cfgSources[$source])) {
-        return PEAR::raiseError(_("Invalid address book"), 'horde.error', null, null, $source);
+        return PEAR::raiseError(sprintf(_("Invalid address book: %s"), $source), 'horde.error', null, null, $source);
     }
 
     $storage = &Turba_Driver::singleton($source);
     if (is_a($storage, 'PEAR_Error')) {
-        return PEAR::raiseError(sprintf(_("Connection failed: %s"), $driver->getMessage()), 'horde.error', null, null, $source);
+        return PEAR::raiseError(sprintf(_("Connection failed: %s"), $storage->getMessage()), 'horde.error', null, null, $source);
     }
 
     $results = $storage->search(array());
 
     if (is_a($results, 'PEAR_Error')) {
-        return PEAR::raiseError(sprintf(_("Error searching the address book: %s"), $driver->getMessage()), 'horde.error', null, null, $source);
+        return PEAR::raiseError(sprintf(_("Error searching the address book: %s"), $results->getMessage()), 'horde.error', null, null, $source);
     }
 
     $r = array();
@@ -433,20 +438,30 @@ function _turba_list($source = '')
  *
  * @return array  An array of UIDs matching the action and time criteria.
  */
-function _turba_listBy($action, $timestamp, $source = '')
+function _turba_listBy($action, $timestamp, $source = null)
 {
-    global $prefs;
+    global $prefs, $cfgSources;
     require_once dirname(__FILE__) . '/base.php';
 
+    // FIXME! Turba stores username or owner in history, not the address book source
+    // So we have to search for username rather than source.
+    $source = Auth::getAuth();
+    /*
     // Get default address book from user preferences.
     if (empty($source)) {
         $source = $prefs->getValue('default_dir');
+        // On new installations default_dir is not set.
+        // Use first source instead.
+        if (empty($source)) {
+            $source = array_keys($cfgSources);
+            $source = $source[0];
+        }
     }
 
     if (empty($source) || !isset($cfgSources[$source])) {
-        return PEAR::raiseError(_("Invalid address book"), 'horde.error', null, null, $source);
+        return PEAR::raiseError(sprintf(_("Invalid address book: %s"), $source), 'horde.error', null, null, $source);
     }
-
+    */
     $history = &Horde_History::singleton();
     $histories = $history->getByTimestamp('>', $timestamp, array(array('op' => '=', 'field' => 'action', 'value' => $action)), 'turba:' . $source);
     if (is_a($histories, 'PEAR_Error')) {
@@ -466,21 +481,31 @@ function _turba_listBy($action, $timestamp, $source = '')
  *
  * @return integer  The timestamp for this action.
  */
-function _turba_getActionTimestamp($uid, $action, $source = '')
+function _turba_getActionTimestamp($uid, $action, $source = null)
 {
     global $prefs;
     require_once dirname(__FILE__) . '/base.php';
     $history = &Horde_History::singleton();
 
+    // FIXME! Turba stores username or owner in history, not the address book source
+    // So we have to search for username rather than adre
+    $source = Auth::getAuth();
+    /*
     // Get default address book from user preferences.
     if (empty($source)) {
         $source = $prefs->getValue('default_dir');
+        // On new installations default_dir is not set.
+        // Use first source instead.
+        if (empty($source)) {
+            $source = array_keys($cfgSources);
+            $source = $source[0];
+        }
     }
 
     if (empty($source) || !isset($cfgSources[$source])) {
-        return PEAR::raiseError(_("Invalid address book"), 'horde.error', null, null, $source);
+        return PEAR::raiseError(sprintf(_("Invalid address book: %s"), $source), 'horde.error', null, null, $source);
     }
-
+    */
     return $history->getActionTimestamp('turba:' . $source . ':' . $uid, $action);
 }
 
@@ -495,19 +520,25 @@ function _turba_getActionTimestamp($uid, $action, $source = '')
  *
  * @return string  The new UID, or false on failure.
  */
-function _turba_import($content, $contentType = 'array', $source = '')
+function _turba_import($content, $contentType = 'array', $source = null)
 {
     require_once dirname(__FILE__) . '/base.php';
     global $cfgSources, $prefs;
 
-    // Get default address book from user preferences.
+    /* Get default address book from user preferences. */
     if (empty($source)) {
         $source = $prefs->getValue('default_dir');
+        /* On new installations default_dir is not set. */
+        /* Use first source instead. */
+        if (empty($source)) {
+            $source = array_keys($cfgSources);
+            $source = $source[0];
+        }
     }
 
     // Check existance of and permissions on the specified source.
     if (!isset($cfgSources[$source])) {
-        return PEAR::raiseError(_("Invalid address book"), 'horde.warning');
+        return PEAR::raiseError(sprintf(_("Invalid address book: %s"), $source), 'horde.warning');
     }
 
     $driver = &Turba_Driver::singleton($source);
@@ -526,7 +557,7 @@ function _turba_import($content, $contentType = 'array', $source = '')
 
         case 'text/x-vcard':
             require_once 'Horde/iCalendar.php';
-            $iCal = &new Horde_iCalendar();
+            $iCal = new Horde_iCalendar();
             if (!$iCal->parsevCalendar($content)) {
                 return PEAR::raiseError(_("There was an error importing the iCalendar data."));
             }
@@ -592,44 +623,32 @@ function _turba_import($content, $contentType = 'array', $source = '')
  * @param mixed $contentType  What format should the data be in?
  *                            Either a string with one of:
  *                            <pre>
+ *                             text/vcard
  *                             text/x-vcard
  *                            </pre>
- *                            or an array with options:
- *                            <pre>
- *                             'ContentType':  as above
- *                             'ENCODING': (optional) character encoding
- *                                         for strings fields
- *                             'CHARSET':  (optional) charset. Like UTF-8
- *                             'VERSION':  (optional) Format version (ex. iCalendar 2.0, vCalendar 1.0).
- *                            </pre>
- *
+ *                            The first produces a vcard3.0 (rfc2426),
+ *                            the second produces a vcard in old 2.1 format
+ *                            defined by imc.org
  * @return string  The requested data.
  */
-function _turba_export($uid, $contentType, $source = '')
+function _turba_export($uid, $contentType, $source = null)
 {
     require_once dirname(__FILE__) . '/base.php';
     global $cfgSources, $prefs;
 
-    $version = '2.1';
-    if (is_array($contentType)) {
-        $options = $contentType;
-        $contentType = $options['ContentType'];
-        unset($options['ContentType']);
-        if (isset($options['VERSION'])) {
-            $version = $options['VERSION'];
-            unset($options['VERSION']);
-        }
-    } else {
-        $options = array();
-    }
-
-    // Get default address book from user preferences.
+    /* Get default address book from user preferences. */
     if (empty($source)) {
         $source = $prefs->getValue('default_dir');
+        /* On new installations default_dir is not set. */
+        /* Use first source instead. */
+        if (empty($source)) {
+            $source = array_keys($cfgSources);
+            $source = $source[0];
+        }
     }
 
     if (empty($source) || !isset($cfgSources[$source])) {
-        return PEAR::raiseError(_("Invalid address book"), 'horde.error', null, null, $source);
+        return PEAR::raiseError(sprintf(_("Invalid address book: %s"), $source), 'horde.error', null, null, $source);
     }
 
     if (empty($uid)) {
@@ -645,8 +664,6 @@ function _turba_export($uid, $contentType, $source = '')
         return PEAR::raiseError(_("Permission Denied"), 'horde.error', null, null, $source);
     }
 
-    // If the objectId isn't in $source in the first place, just return
-    // true. Otherwise, try to delete it and return success or failure.
     $result = $driver->search(array('__uid' => $uid));
     if (is_a($result, 'PEAR_Error')) {
         return $result;
@@ -657,31 +674,27 @@ function _turba_export($uid, $contentType, $source = '')
         return PEAR::raiseError("Internal Horde Error: multiple turba objects with same objectId.", 'horde.error', null, null, $source);
     }
 
+    $version = '3.0';
     switch ($contentType) {
+    case 'text/x-vcard;version=2.1':
     case 'text/x-vcard':
+        $version = '2.1';
+    case 'text/vcard':
         require_once 'Horde/iCalendar.php';
-        $iCal = &new Horde_iCalendar();
+
         $export = '';
         foreach ($result->objects as $obj) {
-            $vcard = $driver->tovCard($obj);
-            $vcard->setParameter('FN', $options);
-            $vcard->setParameter('ORG', $options);
-            $vcard->setParameter('ADR', $options);
-            $vcard->setParameter('LABEL', $options);
-            $vcard->setParameter('NICKNAME', $options);
-            $vcard->setParameter('TITLE', $options);
-            $vcard->setParameter('NOTE', $options);
-            $vcard->setParameter('N', $options);
-            // vCards are not enclosed in BEGIN:VCALENDAR..END:VCALENDAR so
-            // export the individual cards instead:
+            $vcard = $driver->tovCard($obj, $version);
+            $vcard->setAttribute('VERSION', $version);
+            /* vCards are not enclosed in BEGIN:VCALENDAR..END:VCALENDAR. */
+            /* Export the individual cards instead. */
             $export .= $vcard->exportvCalendar();
-            $iCal->addComponent($vcard);
         }
         return $export;
-
-    default:
-        return PEAR::raiseError(sprintf(_("Unsupported Content-Type: %s"), $contentType));
     }
+
+    return PEAR::raiseError(sprintf(_("Unsupported Content-Type: %s"), $contentType));
+
 }
 
 /**
@@ -692,7 +705,7 @@ function _turba_export($uid, $contentType, $source = '')
  *
  * @return boolean  Success or failure.
  */
-function _turba_delete($uid, $source = '')
+function _turba_delete($uid, $source = null)
 {
     // Handle an array of UIDs for convenience of deleting multiple contacts
     // at once.
@@ -710,13 +723,19 @@ function _turba_delete($uid, $source = '')
     require_once dirname(__FILE__) . '/base.php';
     global $cfgSources, $prefs;
 
-    // Get default address book from user preferences.
+    /* Get default address book from user preferences. */
     if (empty($source)) {
         $source = $prefs->getValue('default_dir');
+        /* On new installations default_dir is not set. */
+        /* Use first source instead. */
+        if (empty($source)) {
+            $source = array_keys($cfgSources);
+            $source = $source[0];
+        }
     }
 
     if (empty($source) || !isset($cfgSources[$source])) {
-        return PEAR::raiseError(_("Invalid address book"), 'horde.error', null, null, $source);
+        return PEAR::raiseError(sprintf(_("Invalid address book: %s"), $source), 'horde.error', null, null, $source);
     }
 
     if (empty($uid)) {
@@ -740,9 +759,8 @@ function _turba_delete($uid, $source = '')
     } elseif ($result->count() == 0) {
         return true;
     } else {
-        // We need to delete by uid rather than by key, so use _delete instead
-        // of delete.
-        return $driver->_delete($driver->toDriver('__uid'), $uid);
+        $r = $result->objects[0];
+        return $driver->delete($r->getValue('__key'));
     }
 }
 
@@ -757,18 +775,24 @@ function _turba_delete($uid, $source = '')
  *
  * @return boolean  Success or failure.
  */
-function _turba_replace($uid, $content, $contentType, $source = '')
+function _turba_replace($uid, $content, $contentType, $source = null)
 {
     require_once dirname(__FILE__) . '/base.php';
     global $cfgSources, $prefs;
 
-    // Get default address book from user preferences.
+    /* Get default address book from user preferences. */
     if (empty($source)) {
         $source = $prefs->getValue('default_dir');
+        /* On new installations default_dir is not set. */
+        /* Use first source instead. */
+        if (empty($source)) {
+            $source = array_keys($cfgSources);
+            $source = $source[0];
+        }
     }
 
     if (!isset($cfgSources) || !is_array($cfgSources) || !count($cfgSources) || !isset($cfgSources[$source])) {
-        return PEAR::raiseError(_("Invalid address book"), 'horde.warning');
+        return PEAR::raiseError(sprintf(_("Invalid address book: %s"), $source), 'horde.warning');
     }
 
     if (empty($uid)) {
@@ -796,7 +820,7 @@ function _turba_replace($uid, $content, $contentType, $source = '')
 
     case 'text/x-vcard':
         require_once 'Horde/iCalendar.php';
-        $iCal = &new Horde_iCalendar();
+        $iCal = new Horde_iCalendar();
         if (!$iCal->parsevCalendar($content)) {
             return PEAR::raiseError(_("There was an error importing the iCalendar data."));
         }
@@ -861,7 +885,7 @@ function _turba_search($names = array(), $sources = array(), $fields = array())
         if (is_a($driver, 'PEAR_Error')) {
             return PEAR::raiseError(sprintf(_("Connection failed: %s"), $driver->getMessage()), 'horde.error', null, null, $source);
         }
- 
+
         // Determine the name of the column to sort by.
         $columns = isset($sort_columns[$source])
             ? $sort_columns[$source] : array();
@@ -891,7 +915,10 @@ function _turba_search($names = array(), $sources = array(), $fields = array())
             while ($ob = $res->next()) {
                 if (!$ob->isGroup()) {
                     /* Not a group. */
-                    $att = $ob->getAttributes();
+                    $att = array('__key' => $ob->getValue('__key'));
+                    foreach ($ob->driver->getCriteria() as $info_key => $info_val) {
+                        $att[$info_key] = $ob->getValue($info_key);
+                    }
                     $email = array();
                     foreach (array_keys($att) as $key) {
                         if ($ob->getValue($key) && isset($attributes[$key]) &&
@@ -1116,9 +1143,10 @@ function _turba_deleteClient($objectId = '')
 
 function _turba_searchClients($names = array(), $fields = array())
 {
-    return $GLOBALS['registry']->call('clients/search', array('names' => $names,
-                                                              'sources' => array($GLOBALS['conf']['client']['addressbook']),
-                                                              'fields' => $fields));
+    return $GLOBALS['registry']->call('clients/search',
+                                      array('names' => $names,
+                                            'sources' => array($GLOBALS['conf']['client']['addressbook']),
+                                            'fields' => $fields));
 }
 
 function _turba_addField($address = '', $name = '', $field = '', $value = '', $source = '')
@@ -1199,7 +1227,7 @@ function _turba_addField($address = '', $name = '', $field = '', $value = '', $s
 }
 
 function _turba_getField($address = '', $field = '', $sources = array(),
-                         $strict = false)
+                         $strict = false, $multiple = false)
 {
     require_once dirname(__FILE__) . '/base.php';
     global $cfgSources;
@@ -1219,7 +1247,7 @@ function _turba_getField($address = '', $field = '', $sources = array(),
         $sources = array(key($cfgSources));
     }
 
-    $count = 0;
+    $result = array();
     foreach ($sources as $source) {
         if (!isset($cfgSources[$source])) {
             continue;
@@ -1237,19 +1265,22 @@ function _turba_getField($address = '', $field = '', $sources = array(),
 
         while ($ob = $list->next()) {
             if ($ob->hasValue($field)) {
-                $count++;
-                $result = $ob->getValue($field);
+                $result[] = $ob->getValue($field);
             }
         }
     }
 
-    if ($count > 1) {
-        return PEAR::raiseError(_("More than 1 entry found"), 'horde.warning', null, null, $source);
-    } elseif (!isset($result)) {
+    if (count($result) > 1) {
+        if ($multiple) {
+            return $result;
+        } else {
+            return PEAR::raiseError(_("More than 1 entry found"), 'horde.warning', null, null, $source);
+        }
+    } elseif (empty($result)) {
         return PEAR::raiseError(sprintf(_("No %s entry found for %s"), $field, $address), 'horde.warning', null, null, $source);
     }
 
-    return $result;
+    return reset($result);
 }
 
 function _turba_deleteField($address = '', $field = '', $sources = array())
