@@ -1,6 +1,6 @@
 <?php
 /**
- * $Horde: turba/minisearch.php,v 1.20.4.7 2006/02/18 03:32:46 chuck Exp $
+ * $Horde: turba/minisearch.php,v 1.20.4.8 2006/05/16 07:42:45 jan Exp $
  *
  * Copyright 2000-2006 Charles J. Hagenbuch <chuck@horde.org>
  *
@@ -66,7 +66,7 @@ if (count($results)) {
                                                        'key' => $contact['key']));
 
         $mail_link = $GLOBALS['registry']->call('mail/compose', array(array('to' => addslashes($contact['email']))));
-        $target = ' target="_parent"';
+        $target = strpos($mail_link, 'javascript:') === 0 ? '' : ' target="_parent"';
         if (is_a($mail_link, 'PEAR_Error')) {
             $mail_link = 'mailto:' . urlencode($contact['email']);
             $target = '';
